#!/usr/bin/python3
# @u:file compteur.py
# @u:File
""" module porposant la classe Compteur """
class Compteur(object):
    """ classe qui permet de compter des éléments """
    def __init__(self, val_init=None):
        self._stat = {}
        if val_init:
            for cle in val_init.keys():
                self._stat[cle] = val_init[cle]

    def incrementer(self, element):
        """ méthode qui incrémente le nom d'un élément """
        if element in self._stat.keys():
            self._stat[element] = self._stat[element] + 1
        else:
            self._stat[element] = 1

    def fixer(self, element, valeur):
        """ méthode qui permet de fixer le nombre d'occurences d'un élément """
        self._stat[valeur] = element

    def nb_occurrences(self, element):
        """ méthode qui permet d'obtenir le nombre d'occurences d'un élément """
        if element not in self._stat.keys():
            return 0
        return self._stat[element]

    @property
    def elements(self):
        """ propriété qui permet d'obtenir tous les éléments référencés """
        return set(self._stat.keys())

    def _elements_ayant_un_nb_occurrences(self,
                                          selection_valeur=lambda l: l[0]):
        if not self._stat:
            return set()
        val = selection_valeur(self._stat.values())
        return {el for el in self._stat if self._stat[el] == val}

    def elements_moins_frequents(self):
        """ fonction qui permet d'obtenir tous les éléments référencés les moins fréquents """
        return self._elements_ayant_un_nb_occurrences(lambda l: min(l))

    def elements_plus_frequents(self):
        """ fonction qui permet d'obtenir tous les éléments référencés les plus fréquents """
        return self._elements_ayant_un_nb_occurrences(lambda l: max(l))

    def elements_par_nb_occurrences(self):
        """ fonction qui permet d'obtenir tous les éléments en fonction
de leurs nombre d'occurences """
        les_nb_occurrences = set([self.nb_occurrences(e) for e in self.elements])
        return {nb_occurrences:\
                set([e for e in self.elements if self.nb_occurrences(e) == nb_occurrences])\
                for nb_occurrences in les_nb_occurrences}

    def __repr__(self):
        return "Compteur(%s)" % repr(self._stat)

    def __str__(self):
        return str(self._stat)

    def __eq__(self, autre):
        if not isinstance(autre, Compteur):
            return False
        return self._stat == autre._stat
